@@ -42,8 +42,16 @@
 #include "apparmor.h"
 #include "audit.h"
 
+#ifdef DBUS_UNIX
+#include <dbus/dbus-sysdeps-unix.h>
+#endif
+
 static BusContext *context;
 
+#ifdef DBUS_WIN
+#include <dbus/dbus-sysdeps-win.h>
+#endif
+
 #ifdef DBUS_UNIX
 
 /* Despite its name and its unidirectional nature, this is actually
@@ -63,6 +71,10 @@ typedef enum
 static void
 signal_handler (int sig)
 {
+  /* Signal handlers that might set errno must save and restore the errno
+   * that the interrupted function might have been relying on. */
+  int saved_errno = errno;
+
   switch (sig)
     {
     case SIGHUP:
@@ -124,10 +136,19 @@ signal_handler (int sig)
           }
       }
       break;
+
+    default:
+      /* can't happen unless this signal handler gets used for a wrong
+       * signal, but keep -Wswitch-default happy */
+      break;
     }
+
+  errno = saved_errno;
 }
 #endif /* DBUS_UNIX */
 
+static void usage (void) _DBUS_GNUC_NORETURN;
+
 static void
 usage (void)
 {
@@ -142,7 +163,13 @@ usage (void)
       " [--introspect]"
       " [--address=ADDRESS]"
       " [--nopidfile]"
+      " [--nosyslog]"
+      " [--syslog]"
+      " [--syslog-only]"
       " [--nofork]"
+#ifdef DBUS_WIN
+      " [--ready-event-handle=value]"
+#endif
 #ifdef DBUS_UNIX
       " [--fork]"
       " [--systemd-activation]"
@@ -151,6 +178,8 @@ usage (void)
   exit (1);
 }
 
+static void version (void) _DBUS_GNUC_NORETURN;
+
 static void
 version (void)
 {
@@ -162,6 +191,8 @@ version (void)
   exit (0);
 }
 
+static void introspect (void) _DBUS_GNUC_NORETURN;
+
 static void
 introspect (void)
 {
@@ -171,7 +202,7 @@ introspect (void)
   if (!_dbus_string_init (&xml))
     goto oom;
 
-  if (!bus_driver_generate_introspect_string (&xml))
+  if (!bus_driver_generate_introspect_string (&xml, TRUE, NULL))
     {
       _dbus_string_free (&xml);
       goto oom;
@@ -183,7 +214,7 @@ introspect (void)
   exit (0);
 
  oom:
-  _dbus_warn ("Can not introspect - Out of memory\n");
+  _dbus_warn ("Can not introspect - Out of memory");
   exit (1);
 }
 
@@ -252,7 +283,7 @@ handle_reload_watch (DBusWatch    *watch,
   if ((reload_pipe[RELOAD_READ_END].fd > 0) &&
       _dbus_read_socket (reload_pipe[RELOAD_READ_END], &str, 1) != 1)
     {
-      _dbus_warn ("Couldn't read from reload pipe.\n");
+      _dbus_warn ("Couldn't read from reload pipe.");
       close_reload_pipe (&watch);
       return TRUE;
     }
@@ -278,7 +309,7 @@ handle_reload_watch (DBusWatch    *watch,
           _DBUS_ASSERT_ERROR_IS_SET (&error);
           _dbus_assert (dbus_error_has_name (&error, DBUS_ERROR_FAILED) ||
                         dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY));
-          _dbus_warn ("Unable to reload configuration: %s\n",
+          _dbus_warn ("Unable to reload configuration: %s",
                       error.message);
           dbus_error_free (&error);
         }
@@ -296,6 +327,7 @@ handle_reload_watch (DBusWatch    *watch,
         loop = bus_context_get_loop (context);
         if (loop != NULL)
           {
+            _dbus_daemon_report_stopping ();
             _dbus_loop_quit (loop);
           }
       }
@@ -319,7 +351,7 @@ setup_reload_pipe (DBusLoop *loop)
   if (!_dbus_socketpair (&reload_pipe[0], &reload_pipe[1],
                          TRUE, &error))
     {
-      _dbus_warn ("Unable to create reload pipe: %s\n",
+      _dbus_warn ("Unable to create reload pipe: %s",
 		  error.message);
       dbus_error_free (&error);
       exit (1);
@@ -331,7 +363,7 @@ setup_reload_pipe (DBusLoop *loop)
 
   if (watch == NULL)
     {
-      _dbus_warn ("Unable to create reload watch: %s\n",
+      _dbus_warn ("Unable to create reload watch: %s",
 		  error.message);
       dbus_error_free (&error);
       exit (1);
@@ -339,7 +371,7 @@ setup_reload_pipe (DBusLoop *loop)
 
   if (!_dbus_loop_add_watch (loop, watch))
     {
-      _dbus_warn ("Unable to add reload watch to main loop: %s\n",
+      _dbus_warn ("Unable to add reload watch to main loop: %s",
 		  error.message);
       dbus_error_free (&error);
       exit (1);
@@ -355,11 +387,8 @@ close_reload_pipe (DBusWatch **watch)
     _dbus_watch_unref (*watch);
     *watch = NULL;
 
-    _dbus_close_socket (reload_pipe[RELOAD_READ_END], NULL);
-    _dbus_socket_invalidate (&reload_pipe[RELOAD_READ_END]);
-
-    _dbus_close_socket (reload_pipe[RELOAD_WRITE_END], NULL);
-    _dbus_socket_invalidate (&reload_pipe[RELOAD_WRITE_END]);
+    _dbus_close_socket (&reload_pipe[RELOAD_READ_END], NULL);
+    _dbus_close_socket (&reload_pipe[RELOAD_WRITE_END], NULL);
 }
 #endif /* DBUS_UNIX */
 
@@ -378,6 +407,34 @@ main (int argc, char **argv)
   dbus_bool_t print_address;
   dbus_bool_t print_pid;
   BusContextFlags flags;
+  void *ready_event_handle;
+
+#ifdef DBUS_UNIX
+  const char *error_str;
+
+  /* Redirect stdin from /dev/null since we will never need it, and
+   * redirect stdout and stderr to /dev/null if not already open.
+   *
+   * We should do this as the very first thing, to ensure that when we
+   * create other file descriptors (for example for epoll, inotify or
+   * a socket), they never get assigned as fd 0, 1 or 2. If they were,
+   * which could happen if our caller had (incorrectly) closed those
+   * standard fds, they'd get closed when we daemonize - for example,
+   * closing our listening socket would stop us listening, and closing
+   * a Linux epoll socket would cause the main loop to fail. */
+  if (!_dbus_ensure_standard_fds (DBUS_FORCE_STDIN_NULL, &error_str))
+    {
+      fprintf (stderr,
+               "dbus-daemon: fatal error setting up standard fds: %s: %s\n",
+               error_str, _dbus_strerror (errno));
+      return 1;
+    }
+
+  /* Set all fds >= 3 close-on-execute. We don't want activated services
+   * to inherit fds we might have inherited from our caller. */
+  _dbus_fd_set_all_close_on_exec ();
+#endif
+  ready_event_handle = NULL;
 
   if (!_dbus_string_init (&config_file))
     return 1;
@@ -416,6 +473,21 @@ main (int argc, char **argv)
         {
           introspect ();
         }
+      else if (strcmp (arg, "--nosyslog") == 0)
+        {
+          flags &= ~BUS_CONTEXT_FLAG_SYSLOG_ALWAYS;
+          flags |= BUS_CONTEXT_FLAG_SYSLOG_NEVER;
+        }
+      else if (strcmp (arg, "--syslog") == 0)
+        {
+          flags &= ~BUS_CONTEXT_FLAG_SYSLOG_NEVER;
+          flags |= BUS_CONTEXT_FLAG_SYSLOG_ALWAYS;
+        }
+      else if (strcmp (arg, "--syslog-only") == 0)
+        {
+          flags &= ~BUS_CONTEXT_FLAG_SYSLOG_NEVER;
+          flags |= (BUS_CONTEXT_FLAG_SYSLOG_ALWAYS|BUS_CONTEXT_FLAG_SYSLOG_ONLY);
+        }
       else if (strcmp (arg, "--nofork") == 0)
         {
           flags &= ~BUS_CONTEXT_FLAG_FORK_ALWAYS;
@@ -440,14 +512,14 @@ main (int argc, char **argv)
         {
           check_two_config_files (&config_file, "system");
 
-          if (!_dbus_append_system_config_file (&config_file))
+          if (!_dbus_get_system_config_file (&config_file))
             exit (1);
         }
       else if (strcmp (arg, "--session") == 0)
         {
           check_two_config_files (&config_file, "session");
 
-          if (!_dbus_append_session_config_file (&config_file))
+          if (!_dbus_get_session_config_file (&config_file))
             exit (1);
         }
       else if (strstr (arg, "--config-file=") == arg)
@@ -554,6 +626,20 @@ main (int argc, char **argv)
         {
           print_pid = TRUE; /* and we'll get the next arg if appropriate */
         }
+#ifdef DBUS_WIN
+      else if (strstr (arg, "--ready-event-handle=") == arg)
+        {
+          const char *desc;
+          desc = strchr (arg, '=');
+          _dbus_assert (desc != NULL);
+          ++desc;
+          if (sscanf (desc, "%p", &ready_event_handle) != 1)
+            {
+              fprintf (stderr, "%s specified, but invalid handle provided\n", arg);
+              exit (1);
+            }
+        }
+#endif
       else
         {
           usage ();
@@ -616,25 +702,26 @@ main (int argc, char **argv)
 
   if (!bus_selinux_pre_init ())
     {
-      _dbus_warn ("SELinux pre-initialization failed\n");
+      _dbus_warn ("SELinux pre-initialization failed");
       exit (1);
     }
 
   if (!bus_apparmor_pre_init ())
     {
-      _dbus_warn ("AppArmor pre-initialization failed: out of memory\n");
+      _dbus_warn ("AppArmor pre-initialization failed: out of memory");
       exit (1);
     }
 
   dbus_error_init (&error);
   context = bus_context_new (&config_file, flags,
-                             &print_addr_pipe, &print_pid_pipe,
+                             &print_addr_pipe, &print_pid_pipe, ready_event_handle,
                              _dbus_string_get_length(&address) > 0 ? &address : NULL,
                              &error);
   _dbus_string_free (&config_file);
+  _dbus_string_free (&address);
   if (context == NULL)
     {
-      _dbus_warn ("Failed to start message bus: %s\n",
+      _dbus_warn ("Failed to start message bus: %s",
                   error.message);
       dbus_error_free (&error);
       exit (1);
@@ -656,6 +743,7 @@ main (int argc, char **argv)
 #endif /* DBUS_UNIX */
 
   _dbus_verbose ("We are on D-Bus...\n");
+  _dbus_daemon_report_ready ();
   _dbus_loop_run (bus_context_get_loop (context));
 
   bus_context_shutdown (context);
