@@ -255,18 +255,6 @@ dbus_bool_t
 _dbus_decrement_fail_alloc_counter (void)
 {
   _dbus_initialize_malloc_debug ();
-#ifdef DBUS_WIN
-  {
-    static dbus_bool_t called = 0;
-
-    if (!called)
-      {
-        _dbus_verbose_raw ("TODO: memory allocation testing errors disabled for now\n");
-        called = 1;
-      }
-    return FALSE;
-  }
-#endif
 
   if (fail_alloc_counter <= 0)
     {
@@ -349,8 +337,8 @@ check_guards (void       *free_block,
   if (free_block != NULL)
     {
       unsigned char *block = ((unsigned char*)free_block) - GUARD_START_OFFSET;
-      size_t requested_bytes = *(dbus_uint32_t*)block;
-      BlockSource source = *(dbus_uint32_t*)(block + 4);
+      size_t requested_bytes = *(dbus_uint32_t *) (void *) block;
+      BlockSource source = *(dbus_uint32_t *) (void *) (block + 4);
       unsigned int i;
       dbus_bool_t failed;
 
@@ -364,7 +352,7 @@ check_guards (void       *free_block,
       i = GUARD_INFO_SIZE;
       while (i < GUARD_START_OFFSET)
         {
-          dbus_uint32_t value = *(dbus_uint32_t*) &block[i];
+          dbus_uint32_t value = *(dbus_uint32_t *) (void *) &block[i];
           if (value != GUARD_VALUE)
             {
               _dbus_warn ("Block of %lu bytes from %s had start guard value 0x%ux at %d expected 0x%x",
@@ -379,7 +367,7 @@ check_guards (void       *free_block,
       i = GUARD_START_OFFSET + requested_bytes;
       while (i < (GUARD_START_OFFSET + requested_bytes + GUARD_END_PAD))
         {
-          dbus_uint32_t value = *(dbus_uint32_t*) &block[i];
+          dbus_uint32_t value = *(dbus_uint32_t *) (void *) &block[i];
           if (value != GUARD_VALUE)
             {
               _dbus_warn ("Block of %lu bytes from %s had end guard value 0x%ux at %d expected 0x%x",
@@ -412,23 +400,23 @@ set_guards (void       *real_block,
     return NULL;
 
   _dbus_assert (GUARD_START_OFFSET + GUARD_END_PAD == GUARD_EXTRA_SIZE);
-  
-  *((dbus_uint32_t*)block) = requested_bytes;
-  *((dbus_uint32_t*)(block + 4)) = source;
+
+  *((dbus_uint32_t *) (void *) block) = requested_bytes;
+  *((dbus_uint32_t *) (void *) (block + 4)) = source;
 
   i = GUARD_INFO_SIZE;
   while (i < GUARD_START_OFFSET)
     {
-      (*(dbus_uint32_t*) &block[i]) = GUARD_VALUE;
-      
+      (*(dbus_uint32_t *) (void *) &block[i]) = GUARD_VALUE;
+
       i += 4;
     }
 
   i = GUARD_START_OFFSET + requested_bytes;
   while (i < (GUARD_START_OFFSET + requested_bytes + GUARD_END_PAD))
     {
-      (*(dbus_uint32_t*) &block[i]) = GUARD_VALUE;
-      
+      (*(dbus_uint32_t *) (void *) &block[i]) = GUARD_VALUE;
+
       i += 4;
     }
   
