@@ -103,7 +103,7 @@ handle_new_client_fd_and_unlock (DBusServer *server,
   transport = _dbus_transport_new_for_socket (client_fd, &server->guid_hex, NULL);
   if (transport == NULL)
     {
-      _dbus_close_socket (client_fd, NULL);
+      _dbus_close_socket (&client_fd, NULL);
       SERVER_UNLOCK (server);
       return FALSE;
     }
@@ -242,10 +242,7 @@ socket_disconnect (DBusServer *server)
         }
 
       if (_dbus_socket_is_valid (socket_server->fds[i]))
-        {
-          _dbus_close_socket (socket_server->fds[i], NULL);
-          _dbus_socket_invalidate (&socket_server->fds[i]);
-        }
+        _dbus_close_socket (&socket_server->fds[i], NULL);
     }
 
   if (socket_server->socket_name != NULL)
@@ -513,7 +510,7 @@ failed:
   if (listen_fds != NULL)
     {
       for (i = 0; i < nlisten_fds; i++)
-        _dbus_close_socket (listen_fds[i], NULL);
+        _dbus_close_socket (&listen_fds[i], NULL);
       dbus_free (listen_fds);
     }
 
@@ -596,5 +593,287 @@ _dbus_server_socket_own_filename (DBusServer *server,
   socket_server->socket_name = filename;
 }
 
+/**
+ * Creates a new server listening on the given Unix domain socket.
+ *
+ * @param path the path for the domain socket.
+ * @param abstract #TRUE to use abstract socket namespace
+ * @param error location to store reason for failure.
+ * @returns the new server, or #NULL on failure.
+ */
+DBusServer*
+_dbus_server_new_for_domain_socket (const char     *path,
+                                    dbus_bool_t     abstract,
+                                    DBusError      *error)
+{
+  DBusServer *server;
+  DBusSocket listen_fd;
+  DBusString address;
+  char *path_copy;
+  DBusString path_str;
+
+  _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+
+  if (!_dbus_string_init (&address))
+    {
+      dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
+      return NULL;
+    }
+
+  _dbus_string_init_const (&path_str, path);
+  if ((abstract &&
+       !_dbus_string_append (&address, "unix:abstract=")) ||
+      (!abstract &&
+       !_dbus_string_append (&address, "unix:path=")) ||
+      !_dbus_address_append_escaped (&address, &path_str))
+    {
+      dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
+      goto failed_0;
+    }
+
+  if (abstract)
+    {
+      path_copy = NULL;
+    }
+  else
+    {
+      path_copy = _dbus_strdup (path);
+      if (path_copy == NULL)
+        {
+          dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
+          goto failed_0;
+        }
+    }
+
+  listen_fd = _dbus_listen_unix_socket (path, abstract, error);
+
+  if (!_dbus_socket_is_valid (listen_fd))
+    {
+      _DBUS_ASSERT_ERROR_IS_SET (error);
+      goto failed_1;
+    }
+
+  server = _dbus_server_new_for_socket (&listen_fd, 1, &address, 0, error);
+  if (server == NULL)
+    {
+      goto failed_2;
+    }
+
+  if (path_copy != NULL)
+    _dbus_server_socket_own_filename(server, path_copy);
+
+  _dbus_string_free (&address);
+
+  return server;
+
+ failed_2:
+  _dbus_close_socket (&listen_fd, NULL);
+ failed_1:
+  dbus_free (path_copy);
+ failed_0:
+  _dbus_string_free (&address);
+
+  return NULL;
+}
+
+/**
+ * Creates a new Unix domain socket server listening under the given directory.
+ * This function is used for "unix:dir/tmpdir" kind of addresses.
+ *
+ * @param dir the path to a directory.
+ * @param error location to store reason for failure.
+ * @returns the new server, or #NULL on failure.
+ */
+static DBusServer *
+_dbus_server_new_for_dir (const char       *dir,
+                          DBusError        *error)
+{
+  DBusServer *server;
+  DBusString full_path;
+  DBusString filename;
+
+  if (!_dbus_string_init (&full_path))
+    {
+      dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
+      return NULL;
+    }
+
+  if (!_dbus_string_init (&filename))
+    {
+      _dbus_string_free (&full_path);
+      dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
+      return NULL;
+    }
+
+  if (!_dbus_string_append (&filename, "dbus-"))
+    {
+      _dbus_string_free (&full_path);
+      _dbus_string_free (&filename);
+      dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
+      return NULL;
+    }
+
+  if (!_dbus_generate_random_ascii (&filename, 10, error))
+    {
+      _dbus_string_free (&full_path);
+      _dbus_string_free (&filename);
+      return NULL;
+    }
+
+  if (!_dbus_string_append (&full_path, dir) ||
+      !_dbus_concat_dir_and_file (&full_path, &filename))
+    {
+      _dbus_string_free (&full_path);
+      _dbus_string_free (&filename);
+      dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
+      return NULL;
+    }
+
+  server =
+    _dbus_server_new_for_domain_socket (_dbus_string_get_const_data (&full_path),
+                                        FALSE,  /* not abstract */
+                                        error);
+
+  _dbus_string_free (&full_path);
+  _dbus_string_free (&filename);
+
+  return server;
+}
+
+/**
+ * Tries to interpret the address entry for UNIX socket
+ * addresses.
+ *
+ * Sets error if the result is not OK.
+ *
+ * @param entry an address entry
+ * @param server_p location to store a new DBusServer, or #NULL on failure.
+ * @param error location to store rationale for failure on bad address
+ * @returns the outcome
+ *
+ */
+DBusServerListenResult
+_dbus_server_listen_unix_socket (DBusAddressEntry *entry,
+                                 DBusServer      **server_p,
+                                 DBusError        *error)
+{
+  const char *method;
+
+  *server_p = NULL;
+
+  method = dbus_address_entry_get_method (entry);
+
+  if (strcmp (method, "unix") == 0)
+    {
+      const char *path = dbus_address_entry_get_value (entry, "path");
+      const char *dir = dbus_address_entry_get_value (entry, "dir");
+      const char *tmpdir = dbus_address_entry_get_value (entry, "tmpdir");
+      const char *abstract = dbus_address_entry_get_value (entry, "abstract");
+      const char *runtime = dbus_address_entry_get_value (entry, "runtime");
+      int mutually_exclusive_modes = 0;
+
+      mutually_exclusive_modes = (path != NULL) + (tmpdir != NULL) +
+        (abstract != NULL) + (runtime != NULL) + (dir != NULL);
+
+      if (mutually_exclusive_modes < 1)
+        {
+          _dbus_set_bad_address(error, "unix",
+                                "path or tmpdir or abstract or runtime or dir",
+                                NULL);
+          return DBUS_SERVER_LISTEN_BAD_ADDRESS;
+        }
+
+      if (mutually_exclusive_modes > 1)
+        {
+          _dbus_set_bad_address(error, NULL, NULL,
+                                "cannot specify two of \"path\", \"tmpdir\", \"abstract\", \"runtime\" and \"dir\" at the same time");
+          return DBUS_SERVER_LISTEN_BAD_ADDRESS;
+        }
+
+      if (runtime != NULL)
+        {
+          DBusString full_path;
+          DBusString filename;
+          const char *runtimedir;
+
+          if (strcmp (runtime, "yes") != 0)
+            {
+              _dbus_set_bad_address(error, NULL, NULL,
+                  "if given, the only value allowed for \"runtime\" is \"yes\"");
+              return DBUS_SERVER_LISTEN_BAD_ADDRESS;
+            }
+
+          runtimedir = _dbus_getenv ("XDG_RUNTIME_DIR");
+
+          if (runtimedir == NULL)
+            {
+              dbus_set_error (error,
+                  DBUS_ERROR_NOT_SUPPORTED, "\"XDG_RUNTIME_DIR\" is not set");
+              return DBUS_SERVER_LISTEN_DID_NOT_CONNECT;
+            }
+
+          _dbus_string_init_const (&filename, "bus");
+
+          if (!_dbus_string_init (&full_path))
+            {
+              _DBUS_SET_OOM (error);
+              return DBUS_SERVER_LISTEN_DID_NOT_CONNECT;
+            }
+
+          if (!_dbus_string_append (&full_path, runtimedir) ||
+              !_dbus_concat_dir_and_file (&full_path, &filename))
+            {
+              _dbus_string_free (&full_path);
+              _DBUS_SET_OOM (error);
+              return DBUS_SERVER_LISTEN_DID_NOT_CONNECT;
+            }
+
+          /* We can safely use filesystem sockets in the runtime directory,
+           * and they are preferred because they can be bind-mounted between
+           * Linux containers. */
+          *server_p = _dbus_server_new_for_domain_socket (
+              _dbus_string_get_const_data (&full_path),
+              FALSE, error);
+
+          _dbus_string_free (&full_path);
+        }
+      else if (tmpdir != NULL || dir != NULL)
+        {
+          /* tmpdir is now equivalent to dir. Previously it would try to
+           * use an abstract socket. */
+          if (tmpdir != NULL)
+            dir = tmpdir;
+
+          *server_p = _dbus_server_new_for_dir (dir, error);
+        }
+      else
+        {
+          if (path)
+            *server_p = _dbus_server_new_for_domain_socket (path, FALSE, error);
+          else
+            *server_p = _dbus_server_new_for_domain_socket (abstract, TRUE, error);
+        }
+
+      if (*server_p != NULL)
+        {
+          _DBUS_ASSERT_ERROR_IS_CLEAR(error);
+          return DBUS_SERVER_LISTEN_OK;
+        }
+      else
+        {
+          _DBUS_ASSERT_ERROR_IS_SET(error);
+          return DBUS_SERVER_LISTEN_DID_NOT_CONNECT;
+        }
+    }
+  else
+    {
+      /* If we don't handle the method, we return NULL with the
+       * error unset
+       */
+      _DBUS_ASSERT_ERROR_IS_CLEAR(error);
+      return DBUS_SERVER_LISTEN_NOT_HANDLED;
+    }
+}
+
 
 /** @} */
