@@ -31,6 +31,8 @@
 
 #include <stdarg.h>
 
+#include <dbus/dbus-macros-internal.h>
+
 DBUS_BEGIN_DECLS
 
 /**
@@ -105,6 +107,8 @@ void          _dbus_string_lock                  (DBusString        *str);
 DBUS_PRIVATE_EXPORT
 dbus_bool_t   _dbus_string_compact               (DBusString        *str,
                                                   int                max_waste);
+DBUS_PRIVATE_EXPORT
+int           _dbus_string_get_allocated_size    (const DBusString *str);
 #ifndef _dbus_string_get_data
 DBUS_PRIVATE_EXPORT
 char*         _dbus_string_get_data              (DBusString        *str);
@@ -215,6 +219,10 @@ DBUS_PRIVATE_EXPORT
 dbus_bool_t   _dbus_string_append_byte           (DBusString        *str,
                                                   unsigned char      byte);
 DBUS_PRIVATE_EXPORT
+dbus_bool_t _dbus_string_append_strings          (DBusString        *str,
+                                                  char             **strings,
+                                                  char               separator);
+DBUS_PRIVATE_EXPORT
 dbus_bool_t   _dbus_string_append_printf         (DBusString        *str,
                                                   const char        *format,
                                                   ...) _DBUS_GNUC_PRINTF (2, 3);
@@ -352,6 +360,10 @@ DBUS_PRIVATE_EXPORT
 void          _dbus_string_chop_white            (DBusString        *str); 
 dbus_bool_t   _dbus_string_append_byte_as_hex    (DBusString        *str,
                                                   unsigned char      byte);
+DBUS_EMBEDDED_TESTS_EXPORT
+dbus_bool_t _dbus_string_append_buffer_as_hex    (DBusString        *str,
+                                                  void              *buf,
+                                                  int                size);
 DBUS_PRIVATE_EXPORT
 dbus_bool_t   _dbus_string_hex_encode            (const DBusString  *source,
                                                   int                start,
