@@ -565,53 +565,6 @@ _dbus_file_exists (const char *file)
   return (access (file, F_OK) == 0);
 }
 
-/** Checks if user is at the console
-*
-* @param username user to check
-* @param error return location for errors
-* @returns #TRUE is the user is at the consolei and there are no errors
-*/
-dbus_bool_t 
-_dbus_user_at_console (const char *username,
-                       DBusError  *error)
-{
-#ifdef DBUS_CONSOLE_AUTH_DIR
-  DBusString u, f;
-  dbus_bool_t result;
-
-  result = FALSE;
-  if (!_dbus_string_init (&f))
-    {
-      _DBUS_SET_OOM (error);
-      return FALSE;
-    }
-
-  if (!_dbus_string_append (&f, DBUS_CONSOLE_AUTH_DIR))
-    {
-      _DBUS_SET_OOM (error);
-      goto out;
-    }
-
-  _dbus_string_init_const (&u, username);
-
-  if (!_dbus_concat_dir_and_file (&f, &u))
-    {
-      _DBUS_SET_OOM (error);
-      goto out;
-    }
-
-  result = _dbus_file_exists (_dbus_string_get_const_data (&f));
-
- out:
-  _dbus_string_free (&f);
-
-  return result;
-#else
-  return FALSE;
-#endif
-}
-
-
 /**
  * Checks whether the filename is an absolute path
  *
@@ -992,16 +945,14 @@ _dbus_parse_unix_group_from_config (const DBusString  *groupname,
  * @param uid the UID
  * @param group_ids return location for array of group IDs
  * @param n_group_ids return location for length of returned array
- * @param error error location
  * @returns #TRUE if the UID existed and we got some credentials
  */
 dbus_bool_t
 _dbus_unix_groups_from_uid (dbus_uid_t            uid,
                             dbus_gid_t          **group_ids,
-                            int                  *n_group_ids,
-                            DBusError            *error)
+                            int                  *n_group_ids)
 {
-  return _dbus_groups_from_uid (uid, group_ids, n_group_ids, error);
+  return _dbus_groups_from_uid (uid, group_ids, n_group_ids);
 }
 
 /**
