@@ -746,7 +746,7 @@ _dbus_string_test (const char *test_data_dir _DBUS_GNUC_UNUSED)
       _dbus_test_fatal ("failed to compact after set_length");
 
     /* peek inside to make sure it worked */
-    if (((DBusRealString *)&str)->allocated > 30)
+    if (_dbus_string_get_allocated_size (&str) > 30)
       _dbus_test_fatal ("compacting string didn't do anything");
 
     if (!_dbus_string_equal_c_str (&str, "abcdefghij"))
