@@ -37,13 +37,15 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
+#include "dbus/dbus-internals.h"
+
 Display *xdisplay = NULL;
 static Atom selection_atom;
 static Atom address_atom;
 static Atom pid_atom;
 
 static int
-x_io_error_handler (Display *xdisplay)
+x_io_error_handler (Display *local_xdisplay)
 {
   verbose ("X IO error\n");
   kill_bus_and_exit (0);
@@ -52,7 +54,7 @@ x_io_error_handler (Display *xdisplay)
 
 static void
 remove_prefix (char *s,
-               char *prefix)
+               const char *prefix)
 {
   int plen;
 
@@ -142,10 +144,15 @@ get_session_file (void)
 
   /* replace the : in the display with _ if the : is still there.
    * use _ instead of - since it can't be in hostnames.
+   *
+   * similarly, on recent versions of macOS, X11 is provided by the XQuartz
+   * package which uses a path for the hostname, such as
+   *   /private/tmp/com.apple.launchd.mBSMLJ3yQU/org.macosforge.xquartz
+   * we therefore also replace any / with _
    */
   for (p = display; *p; ++p)
     {
-      if (*p == ':')
+      if (*p == ':' || *p == '/')
         *p = '_';
     }
   
@@ -248,7 +255,7 @@ init_x_atoms (Display *display)
   user = getpwuid (getuid ());
   if (user == NULL)
     {
-      verbose ("Could not determine the user informations; aborting X11 integration.\n");
+      verbose ("Could not determine user information; aborting X11 integration.\n");
       return FALSE;
     }
   user_name = xstrdup(user->pw_name);
@@ -293,6 +300,7 @@ init_x_atoms (Display *display)
 int
 x11_get_address (char **paddress, pid_t *pid, long *wid)
 {
+  int result;
   Atom type;
   Window owner;
   int format;
@@ -310,10 +318,10 @@ x11_get_address (char **paddress, pid_t *pid, long *wid)
     *wid = (long) owner;
 
   /* get the bus address */
-  XGetWindowProperty (xdisplay, owner, address_atom, 0, 1024, False,
-                      XA_STRING, &type, &format, &items, &after,
-                      (unsigned char **) &data);
-  if (type == None || after != 0 || data == NULL || format != 8)
+  result = XGetWindowProperty (xdisplay, owner, address_atom, 0, 1024, False,
+                              XA_STRING, &type, &format, &items, &after,
+                              (unsigned char **) &data);
+  if (result != Success || type == None || after != 0 || data == NULL || format != 8)
     return FALSE;               /* error */
 
   *paddress = xstrdup (data);
@@ -323,11 +331,11 @@ x11_get_address (char **paddress, pid_t *pid, long *wid)
   if (pid != NULL)
     {
       *pid = 0;
-      XGetWindowProperty (xdisplay, owner, pid_atom, 0, sizeof pid, False,
-                          XA_CARDINAL, &type, &format, &items, &after,
-                          (unsigned char **) &data);
-      if (type != None && after == 0 && data != NULL && format == 32)
-        *pid = (pid_t) *(long*) data;
+      result = XGetWindowProperty (xdisplay, owner, pid_atom, 0, sizeof pid, False,
+                                   XA_CARDINAL, &type, &format, &items, &after,
+                                   (unsigned char **) &data);
+      if (result == Success && type != None && after == 0 && data != NULL && format == 32)
+        *pid = (pid_t) *(long*) (void*) data;
       XFree (data);
     }
 
@@ -366,7 +374,8 @@ set_address_in_x11(char *address, pid_t pid)
   wid = XCreateWindow (xdisplay, RootWindow (xdisplay, 0), -20, -20, 10, 10,
                        0, CopyFromParent, InputOnly, CopyFromParent,
                        0, NULL);
-  verbose ("Created window %d\n", wid);
+  /* The type of a Window varies, so cast it to something reasonable */
+  verbose ("Created window %lu\n", (unsigned long) wid);
 
   /* Save the property in the window */
   XChangeProperty (xdisplay, wid, address_atom, XA_STRING, 8, PropModeReplace,
@@ -404,6 +413,7 @@ set_address_in_file (char *address, pid_t pid, Window wid)
     return FALSE;
 
   f = fopen (session_file, "w");
+  free (session_file);
   if (f == NULL)
     return FALSE;               /* some kind of error */
   fprintf (f,
@@ -412,7 +422,7 @@ set_address_in_file (char *address, pid_t pid, Window wid)
            "# If the DBUS_SESSION_BUS_ADDRESS environment variable is set, it will\n"
            "# be used rather than this file.\n"
            "# See \"man dbus-launch\" for more details.\n"
-           "DBUS_SESSION_BUS_ADDRESS=%s\n"
+           "DBUS_SESSION_BUS_ADDRESS='%s'\n"
            "DBUS_SESSION_BUS_PID=%ld\n"
            "DBUS_SESSION_BUS_WINDOWID=%ld\n",
            get_machine_uuid (),
@@ -420,7 +430,6 @@ set_address_in_file (char *address, pid_t pid, Window wid)
            address, (long)pid, (long)wid);
 
   fclose (f);
-  free (session_file);
 
   return TRUE;
 }
@@ -444,7 +453,24 @@ x11_save_address (char *address, pid_t pid, long *wid)
 int
 x11_init (void)
 {
-  return open_x11 () != NULL && init_x_atoms (xdisplay);
+  int ok;
+
+  /*
+   * The X11 connection is opened and never closed. Because dbus-launch
+   * forks and continues to run non-trivial code in a forked child, it is
+   * not clear whether (or where) it would be safe to close it; instead, we
+   * leave it open until process exit, at which point the socket is cleaned
+   * up by the kernel.
+   *
+   * Any memory allocated for the X11 connection is only allocated once per
+   * run of dbus-launch, so there's no need to keep track of it, and we can
+   * silence memory leak warnings from AddressSanitizer as uninteresting.
+   */
+  _DBUS_BEGIN_IGNORE_LEAKS;
+  ok = open_x11 () != NULL && init_x_atoms (xdisplay);
+  _DBUS_END_IGNORE_LEAKS;
+
+  return ok;
 }
 
 void
